## -----------------------------------------------------------------------------
library(knitr)
opts_chunk$set(autodep = TRUE, cache = TRUE)

## ---- warning = FALSE, message = FALSE, eval = FALSE--------------------------
#  library(devtools)
#  devtools::install_github('mandymejia/clever')

## ---- warning = FALSE, message = FALSE----------------------------------------
library(clever)
data(Dat1)
data(Dat2)

## -----------------------------------------------------------------------------
dim(Dat1)
dim(Dat2)

## -----------------------------------------------------------------------------
clever.Dat1.kurt.lev = clever(Dat1, choose_PCs='kurtosis', 
                              PCA_trend_filtering=FALSE, verbose=TRUE)
clever.Dat2.kurt.lev = clever(Dat2, choose_PCs='kurtosis', 
                              PCA_trend_filtering=FALSE, verbose=TRUE)

## -----------------------------------------------------------------------------
clever.Dat1.vari.lev = clever(Dat1, choose_PCs = 'variance', 
                              PCA_trend_filtering=FALSE)
clever.Dat1.kurt.rds = clever(Dat1, choose_PCs='kurtosis', method = 'robdist_subset', 
                              PCA_trend_filtering=FALSE)
clever.Dat1.vari.rds = clever(Dat1, choose_PCs = 'variance', method = 'robdist_subset', 
                              PCA_trend_filtering=FALSE)
clever.Dat1.kurt.rbd = clever(Dat1, choose_PCs='kurtosis', method = 'robdist', 
                              PCA_trend_filtering=FALSE)
clever.Dat1.vari.rbd = clever(Dat1, choose_PCs = 'variance', method = 'robdist', 
                              PCA_trend_filtering=FALSE)

clever.Dat2.vari.lev = clever(Dat2, choose_PCs = 'variance', 
                              PCA_trend_filtering=FALSE)
clever.Dat2.kurt.rds = clever(Dat2, choose_PCs='kurtosis', method = 'robdist_subset', 
                              PCA_trend_filtering=FALSE)
clever.Dat2.vari.rds = clever(Dat2, choose_PCs = 'variance', method = 'robdist_subset', 
                              PCA_trend_filtering=FALSE)
clever.Dat2.kurt.rbd = clever(Dat2, choose_PCs='kurtosis', method = 'robdist', 
                              PCA_trend_filtering=FALSE)
clever.Dat2.vari.rbd = clever(Dat2, choose_PCs = 'variance', method = 'robdist', 
                              PCA_trend_filtering=FALSE)

clevers.Dat1 = list(clever.Dat1.kurt.lev, clever.Dat1.vari.lev, clever.Dat1.kurt.rds, clever.Dat1.vari.rds, clever.Dat1.kurt.rbd, clever.Dat1.vari.rbd)

clevers.Dat2 = list(clever.Dat2.kurt.lev, clever.Dat2.vari.lev, clever.Dat2.kurt.rds, clever.Dat2.vari.rds, clever.Dat2.kurt.rbd, clever.Dat2.vari.rbd)

## ----warning=FALSE, message=FALSE---------------------------------------------
library(ggpubr)
library(gridExtra)
ggplot2::theme_set(ggpubr::theme_pubr())

## ----fig.height=12, fig.width=7-----------------------------------------------
plt_cell = vector('list', length=6)
plt_row = vector('list', length=3)
for(j in 0:2){
  for(i in 1:2){
    plt_cell[[j*2 + i]] = plot(clevers.Dat1[[j*2 + i]], type='n')
  }
  r = c(plt_cell[(j*2+1):(j*2+2)], common.legend=T, legend='bottom', align='hv', ncol=2)
  plt_row[[j + 1]] = do.call(ggarrange, r)
}

grid.arrange(grobs=plt_row, ncol=1)

## ----fig.height=12, fig.width=7-----------------------------------------------
plt_cell = vector('list', length=6)
plt_row = vector('list', length=3)
for(j in 0:2){
  for(i in 1:2){
    plt_cell[[j*2 + i]] = plot(clevers.Dat2[[j*2 + i]], type='n')
  }
  r = c(plt_cell[(j*2+1):(j*2+2)], common.legend=T, legend='bottom', align='hv', ncol=2)
  plt_row[[j + 1]] = do.call(ggarrange, r)
}

grid.arrange(grobs=plt_row, ncol=1)

## -----------------------------------------------------------------------------
clever.Dat1.PCATF.lev = clever(Dat1, PCA_trend_filtering=TRUE)
clever.Dat2.PCATF.lev = clever(Dat2, PCA_trend_filtering=TRUE)

## ----fig.height=4, fig.width=7------------------------------------------------
grid.arrange(grobs=list(
  D1=plot(clever.Dat1.PCATF.lev)+ggtitle('Dat1, PCATF'), 
  D2=plot(clever.Dat2.PCATF.lev)+ggtitle('Dat2, PCATF')), ncol=2)

## ----warning=FALSE------------------------------------------------------------
library(oro.nifti)
library(neurobase)

#'  Selects a timepoint from a volume time series, and returns it after adding
#'  the NIfTI header from the mask onto it.
#' @param VolumeTimeSeries A 4D matrix. Time is on the 4th dimension.
#' @param time The timepoint to select.
#' @param mask The corresponding mask.
#'
#' @return The 3D volume with the NIfTI header from the mask.
Volume_to_NIfTI <- function(VolumeTimeSeries, time, mask){
  vol <- VolumeTimeSeries[,,,time]
  vol <- copyNIfTIHeader(img=mask, arr=vol)
  return(vol)
}

## -----------------------------------------------------------------------------
fname = system.file("extdata", "Dat1_mask.nii.gz", package = "clever")
Mask1 = readNIfTI(fname) #Pitt_0050048 (full of artifacts)
Img1 = Matrix_to_VolumeTimeSeries(Dat1, Mask1)

fname = system.file("extdata", "Dat2_mask.nii.gz", package = "clever")
Mask2 = readNIfTI(fname)
Img2 = Matrix_to_VolumeTimeSeries(Dat2, Mask2)

## ---- fig.width=7, fig.height=4-----------------------------------------------
par(mfrow=c(1,2))
levs = clever.Dat1.kurt.lev$leverage
t_med = order(levs)[ceiling(length(levs)/2)]
t_max = which.max(levs)

image(Img1[,,,t_med], main=paste0('Median leverage (T = ', t_med, ')'))
image(Img1[,,,t_max], main=paste0('Maximum leverage (T = ', t_max, ')'))

## -----------------------------------------------------------------------------
Lev_Img1 = clever.Dat1.kurt.lev$lev_imgs
print('The timepoints meeting the first outlier level threshold:')
row.names(Lev_Img1$mean)


## ---- fig.width=7, fig.height=4-----------------------------------------------
par(mfrow=c(1,2))

Lev_Img1.mean = Matrix_to_VolumeTimeSeries(Lev_Img1$mean, Mask1)
Lev_Img1.top = Matrix_to_VolumeTimeSeries(Lev_Img1$top, Mask1)

image(Lev_Img1.mean[,,,1], main=paste0('Lev. img., mean dir. (T=60)'))
image(Lev_Img1.top[,,,1], main=paste0('Lev. img., top dir. (T=60)'))

